@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1993-2016 John W. Eaton
+Copyright (C) 1993-2017 John W. Eaton
 
 This file is part of Octave.
 
@@ -379,11 +379,14 @@ safe_source_file (const std::string& file_name,
     {
       recover_from_exception ();
     }
-  catch (const octave::execution_exception&)
+  catch (const octave::execution_exception& ex)
     {
-      recover_from_exception ();
+      std::string stack_trace = ex.info ();
 
-      std::cerr << "error: execution exception in " << file_name << std::endl;
+      if (! stack_trace.empty ())
+        std::cerr << stack_trace;
+
+      recover_from_exception ();
     }
 }
 
@@ -802,8 +805,7 @@ namespace octave
       {
         recover_from_exception ();
 
-        std::cerr << "error: unhandled execution exception -- eval failed"
-                  << std::endl;
+        parse_status = 1;
       }
 
     return parse_status;
@@ -1090,4 +1092,3 @@ namespace octave
       }
   }
 }
-
